@@ -1,4 +1,26 @@
- (* $Id: gText.ml,v 1.14 2005/06/13 01:08:15 garrigue Exp $ *)
+(**************************************************************************)
+(*                Lablgtk                                                 *)
+(*                                                                        *)
+(*    This program is free software; you can redistribute it              *)
+(*    and/or modify it under the terms of the GNU Library General         *)
+(*    Public License as published by the Free Software Foundation         *)
+(*    version 2, with the exception described in file COPYING which       *)
+(*    comes with the library.                                             *)
+(*                                                                        *)
+(*    This program is distributed in the hope that it will be useful,     *)
+(*    but WITHOUT ANY WARRANTY; without even the implied warranty of      *)
+(*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *)
+(*    GNU Library General Public License for more details.                *)
+(*                                                                        *)
+(*    You should have received a copy of the GNU Library General          *)
+(*    Public License along with this program; if not, write to the        *)
+(*    Free Software Foundation, Inc., 59 Temple Place, Suite 330,         *)
+(*    Boston, MA 02111-1307  USA                                          *)
+(*                                                                        *)
+(*                                                                        *)
+(**************************************************************************)
+
+ (* $Id: gText.ml 1415 2008-09-04 15:59:15Z ben_99_9 $ *)
 
 open StdLabels
 open Gaux
@@ -307,22 +329,58 @@ class tag_table_signals obj = object
   inherit text_tag_table_sigs
 end
 
-class tag_table obj = 
+class tag_table_skel obj = 
 object
+  val obj = (obj :> text_tag_table)
   method get_oid = Gobject.get_oid obj
   method as_tag_table : text_tag_table = obj
-  method connect = new tag_table_signals obj
   method add =  TagTable.add obj
   method remove =  TagTable.remove obj
   method lookup =  TagTable.lookup obj
   method size = TagTable.get_size obj
 end
 
+class tag_table obj = 
+object 
+  inherit tag_table_skel obj
+  method connect = new tag_table_signals obj
+end
+  
 let tag_table () = 
   new tag_table (TagTable.create [])
 
-class buffer_signals obj = object (self)
-  inherit ['a] gobject_signals obj
+class type buffer_signals_skel_type = 
+  object
+    method apply_tag :
+      callback:(tag -> start:iter -> stop:iter -> unit) -> GtkSignal.id
+    method begin_user_action : callback:(unit -> unit) -> GtkSignal.id
+    method changed : callback:(unit -> unit) -> GtkSignal.id
+    method delete_range :
+      callback:(start:iter -> stop:iter -> unit) -> GtkSignal.id
+    method end_user_action : callback:(unit -> unit) -> GtkSignal.id
+    method insert_child_anchor :
+      callback:(iter -> Gtk.text_child_anchor -> unit) -> GtkSignal.id
+    method insert_pixbuf :
+      callback:(iter -> GdkPixbuf.pixbuf -> unit) -> GtkSignal.id
+    method insert_text : callback:(iter -> string -> unit) -> GtkSignal.id
+    method mark_deleted : callback:(Gtk.text_mark -> unit) -> GtkSignal.id
+    method mark_set :
+      callback:(iter -> Gtk.text_mark -> unit) -> GtkSignal.id
+    method modified_changed : callback:(unit -> unit) -> GtkSignal.id
+    method remove_tag :
+      callback:(tag -> start:iter -> stop:iter -> unit) -> GtkSignal.id
+  end
+
+class type ['b] buffer_signals_type = 
+object ('a)
+  inherit buffer_signals_skel_type
+  method after : 'a
+  method private connect :
+    'c. ('b, 'c) GtkSignal.t -> callback:'c -> GtkSignal.id
+end
+
+class virtual buffer_signals_skel = 
+object(self)
   inherit text_buffer_sigs
   method apply_tag ~callback = 
     self#connect Buffer.S.apply_tag
@@ -350,6 +408,12 @@ class buffer_signals obj = object (self)
         callback (new tag tag) ~start:(new iter start) ~stop:(new iter stop))
 end
 
+class buffer_signals obj = 
+object 
+  inherit ['a] gobject_signals obj
+  inherit buffer_signals_skel
+end
+
 exception No_such_mark of string
 
 type position =
@@ -359,6 +423,8 @@ type position =
 
 class buffer_skel obj = object(self)
   val obj = (obj :> text_buffer)
+  method private obj = obj
+  inherit text_buffer_props
   method get_oid = Gobject.get_oid obj
   method as_buffer = obj
   method line_count = Buffer.get_line_count obj
@@ -594,7 +660,7 @@ class view obj = object
   method connect = new view_signals obj
 end
 
-let view ?buffer =
+let view ?(buffer:buffer option) =
   View.make_params [] ~cont:(
   GContainer.pack_container ~create:(fun pl ->
     let w = match buffer with 
