@@ -20,7 +20,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(* $Id: gtkTree.ml 1369 2007-09-25 02:56:09Z garrigue $ *)
+(* $Id: gtkTree.ml 1455 2009-05-12 10:31:37Z garrigue $ *)
 
 open Gaux
 open Gtk
@@ -275,7 +275,11 @@ module TreeViewColumn = struct
     = "ml_gtk_tree_view_column_get_sort_column_id"
   external set_cell_data_func : 
     [>`treeviewcolumn] obj -> [>`cellrenderer] obj -> 
-    ([`treemodel] obj -> tree_iter -> unit) option -> unit = "ml_gtk_tree_view_column_set_cell_data_func"
+    ([`treemodel] obj -> tree_iter -> unit) option -> unit
+  = "ml_gtk_tree_view_column_set_cell_data_func"
+  external get_button :
+    [>`treeviewcolumn] obj -> Gtk.button obj
+    = "ml_gtk_tree_view_column_get_button"
 end
 
 module TreeView = struct
@@ -343,6 +347,57 @@ module TreeView = struct
     [>`treeview] obj ->
     (Gtk.tree_model -> Gtk.tree_iter -> bool) option -> unit
     = "ml_gtk_tree_view_set_row_separator_func"
+  
+  (** @since GTK 2.12 *)
+  module Tooltip = struct
+    external set_cell :
+      [>`treeview] obj -> Gtk.tooltip ->
+      ?path:Gtk.tree_path ->
+      ?col:tree_view_column obj ->
+      ?cell:[>`cellrenderer] obj ->
+      unit -> unit
+      = "ml_gtk_tree_view_set_tooltip_cell_bc" "ml_gtk_tree_view_set_tooltip_cell"
+    external set_row :
+      [>`treeview] obj -> Gtk.tooltip -> Gtk.tree_path -> unit
+      = "ml_gtk_tree_view_set_tooltip_row"
+    external get_context :
+      [>`treeview] obj ->
+      x: int -> y: int -> kbd: bool ->
+      (int * int * (Gtk.tree_model * Gtk.tree_path * Gtk.tree_iter) option)
+      = "ml_gtk_tree_view_get_tooltip_context"
+    external get_column :
+      [>`treeview] obj -> int
+      = "ml_gtk_tree_view_get_tooltip_column"
+    external set_column :
+      [>`treeview] obj -> int -> unit
+      = "ml_gtk_tree_view_set_tooltip_column"
+  end
+  
+  module Dnd = struct
+    external get_dest_row_at_pos :
+      [>`treeview] obj ->
+      x: int -> y: int ->
+      ( Gtk.tree_path
+      * GtkEnums.tree_view_drop_position ) option
+      = "ml_gtk_tree_view_get_dest_row_at_pos"
+    external enable_model_drag_dest :
+      [>`treeview] obj ->
+      targets: target_entry array ->
+      actions: Gdk.Tags.drag_action list -> unit
+      = "ml_gtk_tree_view_enable_model_drag_dest"
+    external unset_rows_drag_dest :
+      [>`treeview] obj -> unit
+      = "ml_gtk_tree_view_unset_rows_drag_dest"
+    external enable_model_drag_source :
+      [>`treeview] obj ->
+      ?modi: Gdk.Tags.modifier list ->
+      targets: target_entry array ->
+      actions: Gdk.Tags.drag_action list -> unit
+      = "ml_gtk_tree_view_enable_model_drag_source"
+    external unset_rows_drag_source :
+      [>`treeview] obj -> unit
+      = "ml_gtk_tree_view_unset_rows_drag_source"
+  end
 end
 
 module CellRenderer = CellRenderer
@@ -357,6 +412,8 @@ module CellRendererProgress = CellRendererProgress
 
 module CellRendererCombo = CellRendererCombo
 
+module CellRendererAccel = CellRendererAccel
+
 module CellLayout = struct
   include GtkTreeProps.CellLayout
   let pack layout 
@@ -372,3 +429,43 @@ module TreeSortable = TreeSortable
 module TreeModelFilter = TreeModelFilter
 
 module IconView = IconView
+
+module CustomModel = struct
+  
+  (* Do not change the name of this methods: they are hard coded in ml_gtktree.c.
+     They must remain public. *)
+  class virtual ['row,'a,'b,'c] callback = object
+    method virtual custom_encode_iter : 'row -> 'a * 'b * 'c
+    method virtual custom_decode_iter : 'a -> 'b -> 'c -> 'row
+    method virtual custom_n_columns : int
+    method virtual custom_get_column_type : int -> Gobject.g_type
+    method virtual custom_get_iter : Gtk.tree_path -> 'row option
+    method virtual custom_get_path : 'row -> Gtk.tree_path
+    method virtual custom_get_value : 'row -> int -> Gobject.g_value -> unit
+    method virtual custom_iter_next : 'row -> 'row option
+    method virtual custom_iter_children : 'row option -> 'row option
+    method virtual custom_iter_has_child : 'row -> bool
+    method virtual custom_iter_n_children : 'row option -> int
+    method virtual custom_iter_nth_child : 'row option -> int -> 'row option
+    method virtual custom_iter_parent : 'row -> 'row option
+    method custom_ref_node (_:'row) : unit = ()
+    method custom_unref_node (_:'row) : unit = ()
+  end
+  
+  external create : unit -> tree_model_custom = "ml_custom_model_create"
+
+  external register_callback : 
+    tree_model_custom -> ('row,'a,'b,'c) #callback -> unit = 
+    "ml_register_custom_model_callback_object"
+      
+  external custom_row_inserted : tree_model_custom -> Gtk.tree_path -> 'row -> unit =
+    "ml_custom_model_row_inserted"
+  external custom_row_changed : tree_model_custom -> Gtk.tree_path -> 'row -> unit =
+    "ml_custom_model_row_changed"
+  external custom_row_has_child_toggled : tree_model_custom -> Gtk.tree_path -> 'row -> unit =
+    "ml_custom_model_row_has_child_toggled"
+  external custom_row_deleted : tree_model_custom -> 'row -> unit =
+    "ml_custom_model_row_deleted"
+  external custom_rows_reordered : tree_model_custom -> Gtk.tree_path -> 'row option -> int array -> unit =
+    "ml_custom_model_rows_reordered"
+end
