@@ -20,15 +20,18 @@
 /*                                                                        */
 /**************************************************************************/
 
-/* $Id: ml_gdk.c 1369 2007-09-25 02:56:09Z garrigue $ */
+/* $Id: ml_gdk.c 1452 2009-05-08 10:15:38Z garrigue $ */
 
 #include <string.h>
 #include <gdk/gdk.h>
 #if defined(_WIN32) || defined(__MINGW32__)
 #include <gdk/gdkwin32.h>
 #else
+#if defined(HAS_GTKQUARTZ)
+#else
 #include <gdk/gdkx.h>
 #endif
+#endif
 #include <caml/mlvalues.h>
 #include <caml/alloc.h>
 #include <caml/memory.h>
@@ -83,6 +86,19 @@ CAMLprim value ml_test_##conv (value mask, value test) \
 Make_test(GdkModifier_val)
 Make_test(GdkWindowState_val)
 
+/* Platform */
+
+value ml_gdk_get_platform()
+{
+#ifdef GDK_WINDOWING_WIN32
+  return MLTAG_WIN32;
+#elif defined(GDK_WINDOWING_QUARTZ)
+  return MLTAG_QUARTZ;
+#else
+  return MLTAG_X11;
+#endif
+}
+
 /* Colormap */
 
 ML_0 (gdk_colormap_get_system, Val_GdkColormap)
@@ -253,7 +269,7 @@ ML_1 (gdk_window_clear, GdkWindow_val, Unit)
 ML_0 (GDK_ROOT_PARENT, Val_GdkWindow)
 ML_1 (gdk_window_get_parent, GdkWindow_val, Val_GdkWindow)
 
-#if defined(_WIN32) || defined(__CYGWIN__)
+#if defined(_WIN32) || defined(__CYGWIN__) || defined(HAS_GTKQUARTZ)
 CAMLprim value ml_GDK_WINDOW_XWINDOW(value v)
 {
  ml_raise_gdk ("Not available for Win32");
@@ -488,7 +504,7 @@ CAMLprim value copy_xdata (gint format, void *xdata, gulong nitems)
 CAMLprim value ml_gdk_property_get (value window, value property,
                            value length, value pdelete)
 {
-#if defined(_WIN32) || defined(__CYGWIN__)
+#if defined(_WIN32) || defined(__CYGWIN__) || defined(HAS_GTKQUARTZ)
   return Val_unit; /* not supported */
 #else
     GdkAtom atype;
