@@ -1,4 +1,26 @@
-/* $Id: ml_gtklist.c,v 1.9 2005/06/30 09:10:00 garrigue Exp $ */
+/**************************************************************************/
+/*                Lablgtk                                                 */
+/*                                                                        */
+/*    This program is free software; you can redistribute it              */
+/*    and/or modify it under the terms of the GNU Library General         */
+/*    Public License as published by the Free Software Foundation         */
+/*    version 2, with the exception described in file COPYING which       */
+/*    comes with the library.                                             */
+/*                                                                        */
+/*    This program is distributed in the hope that it will be useful,     */
+/*    but WITHOUT ANY WARRANTY; without even the implied warranty of      */
+/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       */
+/*    GNU Library General Public License for more details.                */
+/*                                                                        */
+/*    You should have received a copy of the GNU Library General          */
+/*    Public License along with this program; if not, write to the        */
+/*    Free Software Foundation, Inc., 59 Temple Place, Suite 330,         */
+/*    Boston, MA 02111-1307  USA                                          */
+/*                                                                        */
+/*                                                                        */
+/**************************************************************************/
+
+/* $Id: ml_gtklist.c 1452 2009-05-08 10:15:38Z garrigue $ */
 
 #include <string.h>
 #include <gtk/gtk.h>
@@ -110,7 +132,7 @@ CAMLprim value ml_gtk_clist_get_text (value clist, value row, value column)
     return Val_optstring(text);
 }
 ML_5 (gtk_clist_set_pixmap, GtkCList_val, Int_val, Int_val, GdkPixmap_val,
-      GdkBitmap_val, Unit)
+      (GdkBitmap*)Pointer_val, Unit)
 CAMLprim value ml_gtk_clist_get_pixmap (value clist, value row, value column)
 {
     CAMLparam0 ();
@@ -131,7 +153,7 @@ CAMLprim value ml_gtk_clist_get_pixmap (value clist, value row, value column)
     CAMLreturn(ret);
 }
 ML_7 (gtk_clist_set_pixtext, GtkCList_val, Int_val, Int_val, String_val,
-      (guint8)Long_val, GdkPixmap_val, GdkBitmap_val, Unit)
+      (guint8)Long_val, GdkPixmap_val, (GdkBitmap*)Pointer_val, Unit)
 ML_bc7 (ml_gtk_clist_set_pixtext)
 ML_3 (gtk_clist_set_foreground, GtkCList_val, Int_val, GdkColor_val, Unit)
 ML_3 (gtk_clist_set_background, GtkCList_val, Int_val, GdkColor_val, Unit)
